/*
 *   $Id: defaults.h,v 1.9 2004/10/26 05:30:34 psavola Exp $
 *
 *   Authors:
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996,1997 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lutchann@litech.org>.
 *
 */

#ifndef DEFAULTS_H
#define DEFAULTS_H

#include <config.h>
#include <includes.h>
#include <radvd.h>

/* maximum message size for incoming and outgoing RSs and RAs */

#define MSG_SIZE			4096

#define MAX2(X,Y) ( (( X ) >=  ( Y )) ? ( X ) : ( Y ))
#define MIN2(X,Y) ( (( X ) >   ( Y )) ? ( Y ) : ( X ))


/* Router Configuration Variables: */

/* For each multicast interface: */

#define DFLT_IgnoreIfMissing		0
#define DFLT_AdvSendAdv			0
#define DFLT_MaxRtrAdvInterval		600
#define DFLT_MinRtrAdvInterval(iface)	(0.33 * (iface)->MaxRtrAdvInterval)
#define DFLT_AdvManagedFlag		0
#define DFLT_AdvOtherConfigFlag		0
#define DFLT_AdvLinkMTU			0
#define DFLT_AdvReachableTime		0
#define DFLT_AdvRetransTimer		0
#define DFLT_AdvCurHopLimit		64	/* as per RFC 1700 or the 
						   next incarnation of it :) */
#define DFLT_AdvDefaultLifetime(iface)	MAX2(1, (int)(3.0 * (iface)->MaxRtrAdvInterval))
#define DFLT_MinDelayBetweenRAs		MIN_DELAY_BETWEEN_RAS
#define DFLT_AdvDefaultPreference	0

#define DFLT_RouterToRouter             0    /* draft-daley-dna-det-fastra-01 */
#define DFLT_RouterToRouterStatusInt    15.0 /* draft-daley-dna-det-fastra-01 */
#define DFLT_DetermineFastRA            0    /* draft-daley-dna-det-fastra-01 */

#define DFLT_UseLinkID                  0    /* draft-pentland-mobileip-linkid-02.txt */
#define DFLT_ForceLinkIDChange          0    /* draft-pentland-mobileip-linkid-02.txt */

#define DFLT_LinkIDBrokenRandom         0    

#define BLOCKING_RNG_NAME               "/dev/random"
#define NONBLOCKING_RNG_NAME            "/dev/urandom"
#define MAX_RNG_NAME_SIZE               16

#define DFLT_PrefixLandmarking          0    
#define DFLT_MaxPrefixLandmarks         8    
#define DFLT_InitialOverloadTime	15  /* Seconds before accounting */

/* Options sent with RA */

#define DFLT_AdvSourceLLAddress		1

#define DFLT_NonceResponse		1

/* Probabilistic Fast RA */
#define DFLT_ProbabilisticFastRA	0
#define DFLT_ProbFastRASlot		20      /* milliseconds */
#define DFLT_ProbFastRAProutersOnly     1
#define DFLT_ProbabilisticNumRouters	1


/* Unnegotiated FastRA */
#define DFLT_MaxFastRARouters		10


/* FastRA token refreshing */

#define DFLT_MaxFastResponses		3	/* 3 fastras?? depletion??*/
#define DFLT_NumUnicastFastRAs		0       /* testing */
#define DFLT_MaxUnicastFastRAs		10  
#define DFLT_FastRATokenInterval	1000	/* milliseconds */  

/* separation of fast responses in ranked fastras */
#define DFLT_FastRASeparation		50  /* ms */

/* Process SLLAOs and create static NCEs: */

#define DFLT_RecvSourceLLAddress	0

/* Process Tentative Source Link Layer Address Options */

#define DFLT_RecvTentativeLLAddress	0 /* draft-daley-ipv6-tsllao-01.txt */

/* Each prefix has an associated: */

#define DFLT_AdvValidLifetime		2592000 /* seconds */
#define DFLT_AdvOnLinkFlag		1
#define DFLT_AdvPreferredLifetime	604800 /* seconds */
#define DFLT_AdvAutonomousFlag		1

/* Each route has an associated: */
#define DFLT_AdvRouteLifetime(iface)	(3 * (iface)->MaxRtrAdvInterval)

#define DFLT_AdvRoutePreference		0 /* medium*/

/* Protocol (RFC2461) constants: */

/* Router constants: */

#define MAX_INITIAL_RTR_ADVERT_INTERVAL	16
#define MAX_INITIAL_RTR_ADVERTISEMENTS	3
#define MAX_FINAL_RTR_ADVERTISEMENTS	3
#define MIN_DELAY_BETWEEN_RAS		3
#define MIN_DELAY_BETWEEN_RAS_MIPv6     (30.0/1000.0)
#define MAX_RA_DELAY_TIME		(1000.0/2.0) /* milliseconds */


#define MAX_PROB_FAST_ROUTERS           5
#define MIN_PROB_FAST_ROUTERS           2


/* Host constants: */

#define MAX_RTR_SOLICITATION_DELAY	1
#define RTR_SOLICITATION_INTERVAL	4
#define MAX_RTR_SOLICITATIONS		3

/* Node constants: */

#define MAX_MULTICAST_SOLICIT		3
#define MAX_UNICAST_SOLICIT		3
#define MAX_ANYCAST_DELAY_TIME		1
#define MAX_NEIGHBOR_ADVERTISEMENT	3
#define REACHABLE_TIME			30000 /* milliseconds */
#define RETRANS_TIMER			1000 /* milliseconds */
#define DELAY_FIRST_PROBE_TIME		5
#define MIN_RANDOM_FACTOR		(1.0/2.0)
#define MAX_RANDOM_FACTOR		(3.0/2.0)

/* MAX and MIN (RFC2461), Mobile IPv6 extensions will override if in use */

#define MIN_MaxRtrAdvInterval		4
#define MAX_MaxRtrAdvInterval		1800

#define MIN_MinRtrAdvInterval		3
#define MAX_MinRtrAdvInterval(iface)	(0.75 * (iface)->MaxRtrAdvInterval)

#define MIN_AdvDefaultLifetime(iface)	(MAX2(1,(iface)->MaxRtrAdvInterval))
#define MAX_AdvDefaultLifetime		9000

#define	MIN_AdvLinkMTU			1280

#define MAX_AdvReachableTime		3600000 /* 1 hour in milliseconds */

#define MAX_AdvCurHopLimit		255

#define MAX_PrefixLen			128

/*
 * Mobile IPv6 extensions, off by default
 */

#define DFLT_AdvRouterAddr		0
#define DFLT_AdvHomeAgentFlag		0
#define DFLT_AdvIntervalOpt		0
#define DFLT_AdvHomeAgentInfo		0

/* Option types (defined also at least in glibc 2.2's netinet/icmp6.h) */

#ifndef ND_OPT_RTR_ADV_INTERVAL
#define ND_OPT_RTR_ADV_INTERVAL         7
#endif
#ifndef ND_OPT_HOME_AGENT_INFO
#define ND_OPT_HOME_AGENT_INFO          8
#endif

/* draft-ietf-ipv6-router-selection-02.txt */
/* XXX: not formally assigned, this will be changed! */
#ifndef ND_OPT_ROUTE_INFORMATION
#define  ND_OPT_ROUTE_INFORMATION	9	

/* XXX: some libc's like KAME already had nd_opt_route_info! */
struct nd_opt_route_info_local     /* route information */
  {
    uint8_t   nd_opt_ri_type;
    uint8_t   nd_opt_ri_len;
    uint8_t   nd_opt_ri_prefix_len;
    uint8_t   nd_opt_ri_flags_reserved;
    uint32_t  nd_opt_ri_lifetime;
    struct in6_addr  nd_opt_ri_prefix;
  };

/* the reserved field is 8 bits and we're interested of the middle two: 000xx000 */
#define ND_OPT_RI_PRF_SHIFT	3
#define ND_OPT_RI_PRF_MASK	(3 << ND_OPT_RI_PRF_SHIFT)

#endif

/* draft-daley-ipv6-tsllao-00.txt */
#ifndef ND_OPT_TENTATIVE_SLLA
#define ND_OPT_TENTATIVE_SLLA        0x11 /* Not assigned yet */
#endif


/* Flags */

#ifndef ND_RA_FLAG_HOME_AGENT
#define ND_RA_FLAG_HOME_AGENT		0x20
#endif
#ifndef ND_OPT_PI_FLAG_RADDR
#define ND_OPT_PI_FLAG_RADDR		0x20
#endif

/* TBD probabilistic FastRA flag */
#ifndef ND_RA_FLAG_PROB_FASTRA
#define ND_RA_FLAG_PROB_FASTRA		0x04
#endif

/* TBD unnegotiated FastRA flag */
#ifndef ND_RA_FLAG_FASTRA
#define ND_RA_FLAG_FASTRA		0x04  /* Note reuse of  flag!! GD */
#endif

/* TBD Prefix Landmarking Flags */
#ifndef ND_RA_FLAG_LANDMARK_SEEN
#define ND_RA_FLAG_LANDMARK_SEEN	0x02
#endif

#ifndef ND_RA_FLAG_LANDMARK_INVALID
#define ND_RA_FLAG_LANDMARK_INVALID	0x01
#endif


/* Configurable values */

#define DFLT_HomeAgentPreference	0
#define DFLT_HomeAgentLifetime(iface)	DFLT_AdvDefaultLifetime(iface)


#define PROB_NUM_ROUTERS_ESTIMATE( X ) \
	MIN2( ( MAX2( MIN_PROB_FAST_ROUTERS, ( X ) ) ) , MAX_PROB_FAST_ROUTERS )

/* Other */

#define AdvertisementInterval(iface)	((iface)->MaxRtrAdvInterval))

#define MIN_MinRtrAdvInterval_MIPv6	(3.0/100.0)
#define MIN_MaxRtrAdvInterval_MIPv6	(7.0/100.0)
#define RTR_SOLICITATION_INTERVAL_MIPv6	1 /* Recommended value by MIPv6 */

#define Cautious_MaxRtrAdvInterval      (2.0/10.0)
#define Cautious_MaxRtrAdvInterval_Leeway      (2.0/100.0)

#define MIN_HomeAgentLifetime		1 /* 0 must NOT be used */
#define MAX_HomeAgentLifetime		65520 /* 18.2 hours in secs */

#define MAX_ResponseOptionLen           64  /* octets */

#define MAX_NonceOptionLen		16 /* octets */

#define MAX_Outstanding_Unicast         DFLT_MaxUnicastFastRAs 
					/* unicast RAs scheduled at any time */

#define FREE_Dynamic_Timers             ( MAX_Outstanding_Unicast )
#define USED_Dynamic_Timers             ( ( MAX_Outstanding_Unicast ) + 1 )

/* #define MAX_RTR_SOLICITATIONS This MAY be ignored by MIPv6 */

#define IID_HASH_MASK32			0x00ffffff

#endif
