/*
 *   $Id: $
 *
 *   Authors:
 *    Greg Daley		<Greg.Daley@eng.monash.edu.au>
 *
 *   This software is Copyright 2004 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lutchann@litech.org>.
 *
 */

#ifndef R2ROUTER_H
#define R2ROUTER_H

#include <includes.h>

/* draft-daley-dna-det-fastra-00.txt */

/* ICMPv6 type for router-to-router messages */

#define  ND_ROUTER_TO_ROUTER   191 /*TBD draft-daley-dna-det-fastra-00.txt */

/* router-to-router message codes */
#define  ND_ROUTER_TO_ROUTER_STATUS_REQ  0 
#define  ND_ROUTER_TO_ROUTER_STATUS      1


struct nd_router_to_router
{
     struct icmp6_hdr  nd_r2r_hdr;     
};
#define nd_r2r_type              nd_r2r_hdr.icmp6_type
#define nd_r2r_code              nd_r2r_hdr.icmp6_code
#define nd_r2r_cksum             nd_r2r_hdr.icmp6_cksum
#define nd_r2r_reserved          nd_r2r_hdr.icmp6_data32[0]


/* options for router to router messages */

#define  ND_OPT_DET_FAST_RA    13  /*TBD draft-daley-dna-det-fastra-00.txt */
#define  ND_OPT_LINK_ID        203 /*TBD draft-pentland-mobileip-linkid-02.txt*/

struct nd_opt_detfra             /* Deterministic FastRA option */
  {
    uint8_t   nd_opt_detfra_type;
    uint8_t   nd_opt_detfra_len;
    uint8_t   nd_opt_detfra_rank;
    uint8_t   nd_opt_detfra_modulus;    /* reserved in -00 */
    uint16_t  nd_opt_detfra_fixeddelay; /* need towork this out */
    uint8_t   nd_opt_detfra_separation;
    uint8_t   nd_opt_detfra_relpref;
  };


#define ND_OPT_DETFRA_NOT_FAST_ROUTER   255
#define ND_OPT_DETFRA_BOOTSTRAP_RANK    254

/* Deterministic FastRA Relative Preference values 0-255 (0=good)*/
#define ND_OPT_DETFRA_DEF_REL_PREF      127 

#define ND_OPT_DETFRA_DEF_SEP           50  /* ms */


struct dna_linkid{
    union
      {
        uint8_t  u_lid8[6];
        uint16_t u_lid16[3];
      } dna_lid_u;
#define link_id                 dna_lid_u.u_lid8
#define link_id16               dna_lid_u.u_lid16
}; 

struct nd_opt_linkid             /* Link Identifier Option */
  {
    uint8_t           nd_opt_linkid_type;
    uint8_t           nd_opt_linkid_len;
    struct dna_linkid nd_opt_linkid_id;
  };

#endif /* R2ROUTER_H */
