/*
 *   $Id: radvd.h,v 1.10 2004/10/26 05:30:34 psavola Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996,1997 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lutchann@litech.org>.
 *
 */

#ifndef RADV_H
#define RADV_H

#include <config.h>
#include <includes.h>
#include <defaults.h>
#include <r2router.h>

#define CONTACT_EMAIL	"Nathan Lutchansky <lutchann@litech.org>"

/* for log.c */
#define	L_NONE		0
#define L_SYSLOG	1
#define L_STDERR	2
#define L_LOGFILE	3

#define LOG_TIME_FORMAT "%b %d %H:%M:%S"

struct Interface;

struct timer_lst {
	struct timeval		expires;
	void			(*handler)(void *);
	void *			data;
	struct timer_lst	*next;
	struct timer_lst	*prev;	
        struct Interface        *iface;
};


struct dtimer_list{
	struct timer_lst   *tm;
	struct dtimer_list *next;
        /* other data for dynamic timers...*/
        struct in6_addr    addr;
	uint32_t	   flags;
        unsigned char      options[MAX_NonceOptionLen];
};

struct prtrlist{
    struct in6_addr addr;
    struct timeval expires;
};

struct AdvPrefix;
struct RtrPrefixList;
struct DetFRARouter;

struct  DetFRARouter{
        struct in6_addr  lladdr;
        uint32_t         RankScore;
        uint8_t          CurrRank;
        uint8_t          LastRank;
        uint8_t          RelPref;
        uint8_t          Separation;
        uint16_t         FixedDelay;
        time_t           twoival;
        time_t           LastUpdate;
        struct  DetFRARouter *next;
        struct  DetFRARouter *prev;
};

#define HWADDR_MAX 16

struct Interface {
	char			Name[IFNAMSIZ];	/* interface name */

	struct in6_addr		if_addr;
	int			if_index;

	uint8_t			if_hwaddr[HWADDR_MAX];
	int			if_hwaddr_len;
	int			if_prefix_len;
	int			if_maxmtu;

	int			IgnoreIfMissing;
	int			AdvSendAdvert;
	double			MaxRtrAdvInterval;
	double			MinRtrAdvInterval;
	double			MinDelayBetweenRAs;
	int			AdvManagedFlag;
	int			AdvOtherConfigFlag;
	int			AdvLinkMTU;
	int			AdvReachableTime;
	int			AdvRetransTimer;
	int			AdvCurHopLimit;
	int			AdvDefaultLifetime;
	int			AdvDefaultPreference;
	int			AdvSourceLLAddress;
	int			RecvSourceLLAddress;
	int			RecvTentativeLLAddress;
	int			UnicastOnly;
	int			NonceResponse;

	int			ProbabilisticFastRA;
	int			ProbFastRAProutersOnly;
	int			ProbFastRASlot;
	int			ProbabilisticNumRouters;
	struct prtrlist         ProbFRtrList[MAX_PROB_FAST_ROUTERS -1];

	int			UnnegotiatedFastRA;

	union			{
					uint8_t  b8[8];
					uint16_t b16[4];
					uint32_t b32[2];
	}			FastRAOwnHashBits;

	int			MaxFastRARouters;
	int			CurrFastRARouters;
	struct RtrPrefixList	*RouterList;

	int			MaxFastResponses; /* not implemented for Detfra */
        int                     FastRASeparation;
	struct timeval		LastFastRAToken;
        int			FastRATokenInterval;
        int			NumUnicastFastRAs;
	int			MaxUnicastFastRAs;

	/* Mobile IPv6 extensions */
	int			AdvIntervalOpt;
	int			AdvHomeAgentInfo;
	int			AdvHomeAgentFlag;
	int16_t			HomeAgentPreference;
	uint16_t		HomeAgentLifetime;

        /* Router to Router Extensions */
        int			RouterToRouter;
        double			RouterToRouterStatusInt;
        int			DetermineFastRA; 
        int			UseLinkID; 
        int                     ForceLinkIDChange;/* Needed? */
	struct timeval		LastLinkIdChange;

	struct timer_lst	rtrtm;
	struct timeval		last_rtrmc;
        int                     r2rState;      /* state right?*/
        int                     r2rStateCount; /* state right?*/

        int                     DetFastRARelPref;
        int                     DetFastRASeparation;
        struct                  DetFRARouter dfraSelf;
        struct                  DetFRARouter dfraFirst; 
        struct                  DetFRARouter dfraLast; 

        struct dna_linkid       CurrentLinkID;
        struct dna_linkid       ProspectiveLinkID;
        int			LinkIDBrokenRandom; 

        int			PrefixLandmarking;
        int			MaxPrefixLandmarks;
        int			CurrPrefixLandmarks;
	struct RtrPrefixList	*PrefixLandList;
	int			PrefixLandOverload;
	struct timeval		LandOverloadExpiry;

	struct AdvPrefix	*AdvPrefixList;
	struct AdvRoute		*AdvRouteList;
	struct timer_lst	tm;
	struct timeval		last_multicast;
        int    			responseAwaitLen;    
        unsigned char  		responseAwaitBuf[MAX_ResponseOptionLen]; //GD
	struct Interface	*next;
	struct dtimer_list      dyntimers[MAX_Outstanding_Unicast+2];
	struct timer_lst        uni_tm[MAX_Outstanding_Unicast];

};

struct AdvPrefix {
	struct in6_addr		Prefix;
	int			PrefixLen;
	
	int			AdvOnLinkFlag;
	int			AdvAutonomousFlag;
	uint32_t		AdvValidLifetime;
	uint32_t		AdvPreferredLifetime;

	/* Mobile IPv6 extensions */
        int                     AdvRouterAddr;

	/* 6to4 extensions */
	char			if6to4[IFNAMSIZ];
	int			enabled;

	struct AdvPrefix	*next;
};


/* More-Specific Routes extensions */

struct AdvRoute {
	struct in6_addr		Prefix;
	int			PrefixLen;
	
	int			AdvRoutePreference;
	uint32_t		AdvRouteLifetime;

	struct AdvRoute		*next;
};

/* Mobile IPv6 extensions */

struct AdvInterval {
	uint8_t			type;
	uint8_t			length;
	uint16_t		reserved;
	uint32_t		adv_ival;
};

struct HomeAgentInfo {
	uint8_t			type;
	uint8_t			length;
	uint16_t		reserved;
	int16_t			preference;
	uint16_t		lifetime;
};	

struct RtrPrefixList{
	struct in6_addr		Prefix;
	struct timeval 		expiry;
	int			this_adv;
	unsigned char		hash[8];
};


#if 0
    0                   1                   2                   3 
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
   |     Type      |     Code      |          Checksum             | 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
   | Cur Hop Limit |M|O|H|Prf|R|S|I|       Router Lifetime         | 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
   |                         Reachable Time                        | 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
   |                          Retrans Timer                        | 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
   |   Options ... 
   +-+-+-+-+-+-+-+-+-+-+-+-

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     Type      |    Length     |           Reserved1           |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                           Reserved2                           |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                                                               |
   +                            Prefix                             +
   |                                                               |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

       Type 0x13 Global Prefix Landmark Option

#endif /* 0 */

#ifndef ND_OPT_LANDMARK
#define ND_OPT_LANDMARK 0x13

struct nd_opt_gpl{
	uint8_t  nd_opt_gpl_type;
	uint8_t  nd_opt_gpl_len;
	uint16_t nd_opt_gpl_reserved1;
	uint32_t nd_opt_gpl_reserved2;
	union
	{
		uint8_t  p6_addr8[8];
		uint16_t p6_addr16[4];
		uint32_t p6_addr32[2];
	} nd_opt_gpl_prefix;
};

#endif /* ND_OPT_LANDMARK*/

/* gram.y */
int yyparse(void);

/* scanner.l */
int yylex(void);

/* timer.c */
void set_timer(struct timer_lst *tm, double);
void clear_timer(struct timer_lst *tm);
void init_timer(struct timer_lst *, void (*)(void *), void *); 

/* log.c */
int log_open(int, char *, char*, int);
int flog(int, char *, ...);
int dlog(int, int, char *, ...);
int log_close(void);
int log_reopen(void);
void set_debuglevel(int);
int get_debuglevel(void);

/* device.c */
int setup_deviceinfo(int, struct Interface *);
int check_device(int, struct Interface *);
int setup_linklocal_addr(int, struct Interface *);
int setup_allrouters_membership(int, struct Interface *);
int check_allrouters_membership(int, struct Interface *);
int check_routertorouter_membership(int sock, struct Interface *iface);
int get_v4addr(const char *, unsigned int *);
int bind_rtnlsock(void);
int open_rtnlsock(void);
void install_nc_entry(int, struct Interface *, struct in6_addr *, unsigned char *, int , int);

/* interface.c */
void iface_init_defaults(struct Interface *);
void prefix_init_defaults(struct AdvPrefix *);
void route_init_defaults(struct AdvRoute *, struct Interface *);
int check_iface(struct Interface *);
int check_if_addr(struct Interface *);

/* socket.c */ int open_icmpv6_socket(void);

/* send.c */
void send_ra(int sock, struct Interface *iface, struct in6_addr *dest,
	unsigned char *outopt);
void send_ra_flags(int sock, struct Interface *iface, struct in6_addr *dest,
	unsigned char *outopt, uint32_t flags);
void send_r2r(int sock, struct Interface *iface, struct in6_addr *dest,
	int req);

/* process.c */
void process(int sock, struct Interface *, unsigned char *, int,
	struct sockaddr_in6 *, struct in6_pktinfo *, int);

/* recv.c */
int recv_rs_ra(int, unsigned char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);

/* util.c */
void mdelay(int);
double rand_between(double, double);
void print_addr(struct in6_addr *, char *);
double tv_diff(struct timeval *, struct timeval *);
struct dtimer_list *remove_from_list(struct dtimer_list *, struct timer_lst *);
void insert_into_drtrlist(struct dtimer_list *, struct dtimer_list *);
void insert_into_list(struct dtimer_list *list, struct dtimer_list *entry);
void log_list(struct dtimer_list *list, const char *listname);
int  add_to_prtrlist(struct Interface *iface, struct in6_addr *addr,
        unsigned int lifetime);
void clear_prtrlist(struct Interface *iface, struct timeval *tv);
int  token_bucket_update(struct Interface *iface, struct timeval *tv);
int prefix_mask(struct in6_addr *prefix, struct in6_addr *addr, int plen);
int check_landmark(struct RtrPrefixList *landlist, int curr,
			struct in6_addr *prefix, struct timeval *now);
int add_rtrprefix(struct RtrPrefixList *landlist, int num, int curr,
		struct in6_addr *prefix, uint32_t expires, int hash);
int update_landtimers(struct RtrPrefixList *landlist, int curr,
		uint32_t expires);

int check_before(struct RtrPrefixList *list, int curr,
                                uint32_t *ownhash, struct in6_addr *addr);

/* radvd.c */
void timer_handler(void *data);
void dynamic_handler(void *data);

/* detfra.c */
int r2r_handledetfrao(struct Interface *iface, struct nd_opt_detfra
                        *dfra, struct sockaddr_in6 *addr, int interval);
uint32_t get_detfra_rank(uint32_t relpref, struct in6_addr *lladdr);
void init_detfrouter(struct DetFRARouter *rtr, uint32_t relpref,
                        struct nd_opt_detfra *detfrao, struct in6_addr *addr);
void iface_DetFastRA_init(struct Interface *iface);
int check_routerconf(struct Interface *iface);
int islinkid(struct dna_linkid *linkid);
int linkid_compare(struct dna_linkid *orig, struct dna_linkid *targ);
void r2rtimers(struct Interface *iface);

int r2r_handlelinkid(struct Interface *iface, struct nd_opt_linkid *lid);

#endif
