/* *   $Id: util.c,v 1.3 2001/11/14 19:58:11 lutchann Exp $
 *
 *   Authors:
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996,1997 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lutchann@litech.org>.
 *
 */

#include <config.h>
#include <includes.h>
#include <radvd.h>
               
void
mdelay(int msecs)
{
	struct timeval tv;
                
	tv.tv_sec = (int) msecs / 1000;
	tv.tv_usec = (msecs % 1000) * 1000;
                                
	select(0,(fd_set *)NULL,(fd_set *)NULL,(fd_set *)NULL, &tv);
}

double
rand_between(double lower, double upper)
{
	return ((upper - lower) / (RAND_MAX + 1.0) * rand() + lower);
}

void
print_addr(struct in6_addr *addr, char *str)
{
	const char *res;

	/* XXX: overflows 'str' if it isn't big enough */
	res = inet_ntop(AF_INET6, (void *)addr, str, INET6_ADDRSTRLEN);
	
	if (res == NULL) 
	{
		strcpy(str, "[invalid address]");	
	}
}

/*return number of milliseconds */

double tv_diff(struct timeval *tv1, struct timeval *tv2){
    double sdiff=0.0;
    int sign=0;
   
         /* reduce gap by ms diff if tv1.sec - tv2.sec negative */
    sign=((tv2->tv_sec > tv1->tv_sec)? -1 : 1);
    
    sdiff = ((long int)tv1->tv_sec - (long int)tv2->tv_sec);
    
    sdiff += sign * 
                (((long int)tv1->tv_usec - (long int)tv2->tv_usec) * 0.000001);
    return sdiff;
}


	/* compares two blocks of memory byte by byte in 
           reverse order */
int revcmp(const void *s1, const void *s2, size_t n){
        int ret=0;
        uint8_t *c1, *c2;
        int i; 
        
        if(!s1  || ! s2){
        	dlog(LOG_DEBUG, 5, "revcmp invalid arg: %p %p",s1, s2);
		return 0;
	}		
   
        c1 = (uint8_t *)s1; 
        c2 = (uint8_t *)s2; 
	for(i = n-1; i >= 0; i--){
		if(c1[i] < c2[i])
			ret = -1; 	

		if(c1[i] > c2[i])
			ret = 1; 	
		if(ret){
        		/*dlog(LOG_DEBUG, 5,
				"revcmp i%d c:%d c1[i]:%hhx c2[i]:%hhx:",
				i, ret, c1[i], c2[i]);*/
			return ret;	
		}
			
	}
        /*dlog(LOG_DEBUG, 5, "revcmp i%d cmp:%d",i, ret);*/
        return ret; 
}

struct dtimer_list *remove_from_list(struct dtimer_list  *list,
		 struct timer_lst *tm){
	struct dtimer_list *entry, *tmp=NULL;

        if(list == NULL)
		return NULL;

        if(tm == NULL){ /* any entry will do (free list only) */
              entry=list->next; 
              if(entry){
                    list->next=entry->next;
                    entry->next=NULL;
              }
              return entry;
        }
        tmp=list;
        while(tmp->next !=NULL){
		if(tmp->next->tm == tm){
			entry=tmp->next;
                        tmp->next=entry->next;
                        entry->next=NULL;
                        return entry;
		}
		tmp=tmp->next;
        }
        return NULL;
}


void insert_into_list(struct dtimer_list *list, struct dtimer_list *entry){
	if(!(list && entry))
		return;

	entry->next=list->next;
        list->next=entry;	
}


void log_list(struct dtimer_list *list, const char *listname){

        char emptystring[1]={'\0'};
        char *tmpstr=NULL;
        struct dtimer_list *tmp;

        tmpstr=emptystring;

	if(list==NULL){
		return;
        }
        if (listname){
		tmpstr=(char *)listname;
	}

        dlog(LOG_DEBUG, 5, "Printing %s list: %p n(%p)", tmpstr, 
                           list, list->next);
        
        tmp=list;
	while(tmp != NULL){

        	dlog(LOG_DEBUG, 5, "\t%p tm(%p) n(%p)",  tmp, tmp->tm,
								 tmp->next);
        	tmp=tmp->next;
        }

}







	/* adds a router to the router list (up to maximum) */
        /* no consideration for timeout yet                 */
        /* modifies the number of routers if one added      */
 
int add_to_prtrlist(struct Interface *iface, struct in6_addr *addr,
        unsigned int lifetime)
{
        int i; 
	char addr_str[INET6_ADDRSTRLEN];
	struct timeval tv, offset;


	print_addr(&iface->if_addr, addr_str);
	dlog(LOG_DEBUG, 5, "checking received RA on Interface %s Ifaddr: %s",
				iface->Name, addr_str);
				
        gettimeofday(&tv, NULL);

	/* evil polling. dependent on input*/
        clear_prtrlist(iface, &tv);

	print_addr(addr, addr_str);
	dlog(LOG_DEBUG, 5, "checking received RA ra source: %s",
				addr_str);	

        /* we always consider ourselves to be part fo the router list */
        /* therefore we do not need to be listed!                     */
	if(!memcmp(&iface->if_addr, addr, sizeof(struct in6_addr)))
	{
		return iface->ProbabilisticNumRouters;
	}

	offset.tv_sec = (lifetime * 2) / 1000; 
	offset.tv_usec = ((lifetime * 2 )% 1000) * 1000 ; 

	for(i = 0; i < (iface->ProbabilisticNumRouters -1 ); i++)
	{
		if(!memcmp(addr, &iface->ProbFRtrList[i].addr, sizeof
			(struct in6_addr)))
		{             
			timeradd(&tv, &offset, &iface->ProbFRtrList[i].expires);
			return iface->ProbabilisticNumRouters;
		}
	}
        /* Already seen enough routers     */
	if(i >= (MAX_PROB_FAST_ROUTERS -1 ))
	{
		return iface->ProbabilisticNumRouters;
	}

	memcpy(&iface->ProbFRtrList[i].addr, addr, sizeof (struct in6_addr));
	timeradd(&tv, &offset, &iface->ProbFRtrList[i].expires);

        ++(iface->ProbabilisticNumRouters);

	dlog(LOG_DEBUG, 4, "Router added to interface %s RouterEst: %d l%d.%d", 
			iface->Name, iface->ProbabilisticNumRouters,
			offset.tv_sec, offset.tv_usec );

	return iface->ProbabilisticNumRouters;
}



int token_bucket_update(struct Interface *iface, struct timeval *tv)
{
	/* returns the number of tokens in the bucket for FastRA */
        double difference;
        int new_tokens;
        struct timeval offsettv;
        struct timeval newtokentv;
        /*double offset;*/

        difference = tv_diff(tv,&iface->LastFastRAToken);

        if(difference < (((double)iface->FastRATokenInterval)/1000.0))
	{
		dlog(LOG_DEBUG, 5, "token bucket:  No New tokens (%d)",
				iface->NumUnicastFastRAs);
		return iface->NumUnicastFastRAs;
	}
        
	dlog(LOG_DEBUG, 5, "token bucket:  last(%d.%d) now(%d.%d) %f %f", 
		iface->LastFastRAToken.tv_sec, iface->LastFastRAToken.tv_usec,
		tv->tv_sec, tv->tv_usec, difference, 
		tv_diff(tv,&iface->LastFastRAToken) );

        new_tokens = (int) (difference * 1000)/iface->FastRATokenInterval; 
   
	dlog(LOG_DEBUG, 5, "new tokens:  %d (ival:%d) ", new_tokens,
			iface->FastRATokenInterval);

        offsettv.tv_sec = (iface->FastRATokenInterval * new_tokens)/ 1000; 
        offsettv.tv_usec = ((iface->FastRATokenInterval * new_tokens)% 1000) 
				* 1000; 

        dlog(LOG_DEBUG, 5, "token offset: %ld secs %ld usecs",
			 offsettv.tv_sec, offsettv.tv_usec);

        timeradd(&iface->LastFastRAToken, &offsettv, &newtokentv);

        memcpy(&iface->LastFastRAToken, &newtokentv, sizeof(struct timeval));        
        if(new_tokens >= (iface->MaxUnicastFastRAs - iface->NumUnicastFastRAs))
	{
		iface->NumUnicastFastRAs = iface->MaxUnicastFastRAs;	
	}
	else{
		iface->NumUnicastFastRAs += new_tokens;
	}

        dlog(LOG_DEBUG, 5, "Tokens %d  Max %d  last token time: %d.%d",
			iface->NumUnicastFastRAs, iface->MaxUnicastFastRAs,
			iface->LastFastRAToken.tv_sec,
			iface->LastFastRAToken.tv_usec);

	return iface->NumUnicastFastRAs;
}

void clear_prtrlist(struct Interface *iface, struct timeval *tv)
{
	int i,j,k=0;
        int nr;

	dlog(LOG_DEBUG, 5, "Prtr clear: %d.%d NR: %d", tv->tv_sec, tv->tv_usec,
			iface->ProbabilisticNumRouters);
	for( i = 0;  i < iface->ProbabilisticNumRouters-1; i++)
        {
		k=j=i;
		nr =  iface->ProbabilisticNumRouters;

      
		while((j <  iface->ProbabilisticNumRouters-1) &&
			 (tv_diff(&iface->ProbFRtrList[j].expires, tv ) < 0))
		{
			dlog(LOG_DEBUG, 5, "Prtr expiry: %d.%d, %f", 
				iface->ProbFRtrList[i].expires.tv_sec,
				iface->ProbFRtrList[i].expires.tv_usec,
				tv_diff(&iface->ProbFRtrList[i].expires, tv));
			j++;		
                        nr--; 
		}

                
		dlog(LOG_DEBUG, 5, "Prtr offset: i: %d j:%d, diff%d NR:%d", 
				i, j, j-i, nr);

                /* O(N^2) for small N */

                for(k=i;j < (iface->ProbabilisticNumRouters-1); k++,j++)
		{
			memcpy(&iface->ProbFRtrList[k], &iface->ProbFRtrList[j],
				sizeof(struct prtrlist));
		}
		iface->ProbabilisticNumRouters  = nr; 
	}
}

int prefix_mask(struct in6_addr *prefix, struct in6_addr *addr, int plen){
	uint32_t mask;
        int i=0; 

	while(plen > 32){
		plen -=32;
		prefix->s6_addr32[i] = addr->s6_addr32[i];
		dlog(LOG_DEBUG, 5, "(%d)%2.2hx:%2.2hx", i,
				 ntohs(prefix->s6_addr16[2 * i]),
				 ntohs(prefix->s6_addr16[(2 * i)+1])); 
		i++; 
	}

	mask = 0xffffffff << (32 - plen);	

	prefix->s6_addr32[i] = addr->s6_addr32[i] & htonl(mask);

	dlog(LOG_DEBUG, 5, "(m%d)%2.2hx:%2.2hx -[%d]", i,
			 ntohs(prefix->s6_addr16[2 * i]),
			 ntohs(prefix->s6_addr16[(2 * i)+1]),plen); 

	i++; 

	for( ; i < 4; i++){
		prefix->s6_addr32[i]=0;
		dlog(LOG_DEBUG, 5, "(a%d)%2.2hx:%2.2hx", i,
			 ntohs(prefix->s6_addr16[2 * i]),
			 ntohs(prefix->s6_addr16[(2 * i)+1])); 
	}
	return 0;
}


int add_rtrprefix(struct RtrPrefixList *landlist, int num, int curr,
	 struct in6_addr *prefix, uint32_t expires, int hash){
	struct timeval now, tv, expirestimer;
	unsigned char digest[SHA_DIGEST_LENGTH];
	int i, j, found=0;

	expirestimer.tv_sec=expires;
	expirestimer.tv_usec=0;
	gettimeofday(&now, NULL);
	timeradd(&now, &expirestimer, &tv);

	dlog(LOG_DEBUG,5,"adding landmarks: Prefix expiry: %lu %d (%d.%d) ",
			expires, curr, tv.tv_sec, tv.tv_usec);

	for (i = 0; i < curr ; i++){

		dlog(LOG_DEBUG,5,"landmark loop: %d curr:%d %f x(%d.%d) n(%d.%d)",
			i,curr,tv_diff(&landlist[i].expiry, &now),
			landlist[i].expiry.tv_sec, landlist[i].expiry.tv_usec,
			now.tv_sec, now.tv_usec);



		while ((i < curr) && (tv_diff(&landlist[i].expiry, &now) < 0)){

			dlog(LOG_DEBUG,5,"landmark loop expiry: %d %f",i,
					tv_diff(&now, &landlist[i].expiry));
			
			memset(&landlist[0], 0, sizeof(struct RtrPrefixList));
			curr--;

			for(j = i; j < curr; j++){
				memcpy( &landlist[j], &landlist[j+1],
					sizeof(struct RtrPrefixList));
			}
			memset(&landlist[j], 0, sizeof(struct RtrPrefixList));
		}
		if(!memcmp(&landlist[i].Prefix, prefix, sizeof(struct
						 in6_addr))){

			dlog(LOG_DEBUG,5,"landmark match: %d  e(%d.%d)",i,
					landlist[i].expiry.tv_sec,
					landlist[i].expiry.tv_usec);
			if(tv_diff(&tv , &landlist[i].expiry) > 0){
				memcpy(&landlist[i].expiry, &tv, sizeof (struct
					timeval));
			}
			landlist[i].this_adv=1;
			found=1;
			return curr;
		}
	}

	dlog(LOG_DEBUG,5,"landmark loop exit: %d c:%d n:%d",i,curr, num);
	if(curr == num)
		return 0;
			

	memcpy(&landlist[curr].Prefix, prefix, sizeof(struct in6_addr));
	memcpy(&landlist[curr].expiry, &tv,  sizeof (struct timeval));
	landlist[curr].this_adv=1;
	if(hash){
		SHA1((unsigned char *)&landlist[curr].Prefix, 
			sizeof(struct in6_addr), digest);
		memcpy(&landlist[curr].hash, digest, 8);
		dlog(LOG_DEBUG,5,"hash install: %d  c:%d %8.8lx%8.8lx",i, curr,
				ntohl(((uint32_t *)landlist[curr].hash)[0]),
				ntohl(((uint32_t *)landlist[curr].hash)[1]));
	}
	dlog(LOG_DEBUG,5,"landmark install: %d  c:%d e(%d.%d)",i, curr,
				landlist[curr].expiry.tv_sec,
				landlist[curr].expiry.tv_usec);
	return curr+1;

}

int check_landmark(struct RtrPrefixList *landlist, int curr,
			struct in6_addr *prefix, struct timeval *now){
	int i;

	dlog(LOG_DEBUG,5,"checking landmarks");

	for (i = 0; (i < curr) ; i++){
		dlog(LOG_DEBUG,5,"landmark check loop %d",i);
		if(!memcmp(&landlist[i].Prefix, prefix, sizeof(struct
								in6_addr))){
			dlog(LOG_DEBUG,5,"landmark check match %d %f",i,
					tv_diff(&landlist[i].expiry,now));
			if(tv_diff(&landlist[i].expiry, now) >= 0){
				return 1;		
			}
			else{
				return 0;
			}
		}

	}
	return 0;
}

/* the land timers need to be set to a nearer expiry if that exists */

int update_landtimers(struct RtrPrefixList *landlist, int curr,
						uint32_t expires){
	int i;
	struct timeval now, tv, expirestimer;

	expirestimer.tv_sec=expires;
	expirestimer.tv_usec=0;
	gettimeofday(&now, NULL);
	timeradd(&now, &expirestimer, &tv);

	for(i = 0; i < curr ; i++){
		
		dlog(LOG_DEBUG,5,"landmark update loop",i);
		if(landlist[i].this_adv && (tv_diff(&tv, &landlist[i].expiry)
					 < 0)){
			memcpy(&landlist[i].expiry, &tv, sizeof
				(struct timeval));	
		}
		landlist[i].this_adv=0;
	}
	return i;
}



int check_before(struct RtrPrefixList *list, int curr,
				uint32_t *ownhash, struct in6_addr *addr){
	int count = 0;
	int i;
	uint64_t ownval[8], calcval[8];
	char  addr_str[40];

	if(curr == 0)
		return 0;

	ownval[0] = *((uint64_t *)ownhash) ^ *((uint64_t *)&addr->s6_addr[8]);

	dlog(LOG_DEBUG,5,"own hash: c:%d %8.8lx%8.8lx",curr,
				ntohl(ownhash[0]), ntohl(ownhash[1]));
	print_addr(addr, addr_str);
	dlog(LOG_DEBUG,4,"own xor: %8.8lx%8.8lx %s",
                                ntohl(((uint32_t *)ownval)[0]),
                                ntohl(((uint32_t *)ownval)[1]),
                                addr_str);

	for(i = 0; i < curr; i ++){
		print_addr(&list[i].Prefix, addr_str);
		calcval[0] = *((uint64_t *)list[i].hash) ^ 
				*((uint64_t *)&addr->s6_addr[8]);
		/*calcval[1] = list[i].hash[1] ^ addr->s6_addr32[3];*/
		dlog(LOG_DEBUG,5,"peer   hash: %d %8.8lx%8.8lx %s",i, 
				ntohl(((uint32_t *)list[i].hash)[0]),
				ntohl(((uint32_t *)list[i].hash)[1]),addr_str);
		dlog(LOG_DEBUG,5,"rec pattern: %d %8.8lx%8.8lx",i, 
				ntohl(addr->s6_addr32[2]),
				ntohl(addr->s6_addr32[3]));

		print_addr(addr, addr_str);
		dlog(LOG_DEBUG,4,"peer    xor: %d %8.8lx%8.8lx %s %d",i,
                                ntohl(((uint32_t *)calcval)[0]),
                                ntohl(((uint32_t *)calcval)[1]),
				addr_str,
				revcmp(ownval, calcval, 8));

		if(revcmp(ownval, calcval, 8) < 0 ){
			count++;
		}
	}

	return count;	
}
